# @signature.digital/tools

A package that defines standard tools for working with contracts.

## Install

To install `@signature.digital/tools`, you'll use your preferred package manager. Assuming you're using npm, follow the steps below:

```bash
npm install @signature.digital/tools
```

Or, if you prefer using Yarn:

```bash
yarn add @signature.digital/tools
```

Make sure that you have Node.js installed on your system to use this package. The module is distributed as an ECMAScript Module (ESM), so you'll need a strategy for dealing with ESM in your project if you're using commonjs modules. As usual, ensure that your TypeScript configuration (`tsconfig.json`) has `"module": "ESNext"` or similar settings that support ESM.

## Usage

### Overview

`@signature.digital/tools` provides a set of interfaces and classes that serve as general utilities for working with contracts digitalization, management, and integration. With a focus on adaptability and ease of use in TypeScript projects, its toolset leverages types, interfaces, and utilities from the `@tsclass/tsclass` package to enhance contract structures. This package aims to facilitate the definition and handling of contractual data, enabling the modeling of business relationships and agreements as TypeScript objects.

### Core Concepts

Before diving into the various tools provided by this package, let's establish a few foundational concepts:

1. **Interface Driven Design:**
   The module uses TypeScript interfaces as blueprints for structuring custom contract models. Interfaces such as `IPortableContract`, `IParagraph`, `IRole`, and `IInvolvedParty` are integral to the way data is defined and manipulated.

2. **Modularity:**
   This package allows developers to segment responsibilities and logic by modularizing contract components into roles, paragraphs, parties, and related structures.

3. **Business Context Integration:**
   Through integration with `@tsclass/tsclass`, the module facilitates the inclusion of business-related metadata into contracts, aligning with typical business use cases such as defining involved parties and their roles.

### Usage Scenarios

#### Defining a Contract

A `IPortableContract` is at the heart of this package. It encapsulates crucial details about a contract such as title, context, roles, parties involved, prior related contracts, and structured paragraphs or sections.

```typescript
import { IPortableContract, IRole, IInvolvedParty, IParagraph } from '@signature.digital/tools';
import { tsclass } from '@signature.digital/tools';

const legalRole: IRole = {
  id: 'role-001',
  name: 'Legal Advisor',
  description: 'Provides legal counsel and insights on contractual obligations.'
};

const stakeholderRole: IRole = {
  id: 'role-002',
  name: 'Stakeholder',
  description: 'Has a stake in the contract outcomes and implications.'
};

const legalAdvisorContact: tsclass.business.IContact = {
  email: 'legal@firm.domain',
  address: '456 Legal Ave, Lawyersville'
};

const stakeholderContact: tsclass.business.IContact = {
  email: 'stakeholder@domain.org',
  address: '789 Stakeholder Loop, Business City'
};

const involvedParties: IInvolvedParty[] = [
  {
    role: legalRole.name,
    contact: legalAdvisorContact
  },
  {
    role: stakeholderRole.name,
    contact: stakeholderContact
  }
];

const introductoryParagraph: IParagraph = {
  uniqueId: 'para-001',
  parent: null,
  title: 'Introduction',
  content: 'This section provides an introduction to the contract, detailing its purpose and significance.'
};

const commitmentParagraph: IParagraph = {
  uniqueId: 'para-002',
  parent: introductoryParagraph,
  title: 'Commitments',
  content: 'Describes the commitments of the involved parties.'
};

const contract: IPortableContract = {
  title: 'Digital Service Agreement',
  context: 'Provides a framework of service delivery between involved entities.',
  availableRoles: [legalRole, stakeholderRole],
  involvedParties,
  priorContracts: [],
  paragraphs: [introductoryParagraph, commitmentParagraph]
};

console.log(contract);
```

### Validating Contracts

Regarding validation, this package utilizes `@tsclass/tsclass` to enhance the consistency and robustness of contact data handling. While not embedding specific validation methods, this encourages the integration of validation strategies that developers can extend or implement themselves.

### Extending the Interfaces

Developers are encouraged to extend the interfaces to accommodate additional requirements.

```typescript
interface IExtendedContract extends IPortableContract {
  effectiveDate: Date;
  expirationDate: Date;
}

const extendedContract: IExtendedContract = {
  ...contract,
  effectiveDate: new Date('2023-01-01'),
  expirationDate: new Date('2025-01-01')
};
```

### Handling Prior Contracts

The `priorContracts` property allows you to model contract inheritance or reference, enabling historical traceability or derivations.

```typescript
const previousContract: IPortableContract = {
  title: 'Precedent Service Agreement',
  context: 'Sets a precedent for the current agreement.',
  availableRoles: [legalRole, stakeholderRole],
  involvedParties,
  priorContracts: [],
  paragraphs: []
};

const referencedContract: IPortableContract = {
  ...contract,
  priorContracts: [previousContract]
};

console.log(referencedContract);
```

### Integrating Business Features

The built-in synergy with `@tsclass/tsclass` permits a natural extension to business contact data directly within the contracts.

```typescript
const newInvolvedParty: IInvolvedParty = {
  role: 'Project Manager',
  contact: {
    email: 'manager@domain.com',
    address: '101 Managerial Blvd, Operationscity'
  }
};

contract.involvedParties.push(newInvolvedParty);
```

### Conclusion

The `@signature.digital/tools` package is designed to provide the essential scaffolding necessary to build intricate, business-oriented contract models while ensuring TypeScript's static type safety. Its major strength lies in its extensibility and the clarity it brings to contract structures, making modeling, data management, and integration straightforward for any digital contract-centric application. Adopt these tools to facilitate the standardized crafting of contract-based solutions across your enterprise applications.
undefined